<?php
/**



*http://code.tutsplus.com/tutorials/creating-client-testimonials-with-custom-post-types--wp-30263



*/
add_action('init', 'testimonials_post_type');
function testimonials_post_type()
{
    $labels = array(
        'name' => 'Testimonials',
        'singular_name' => 'Testimonial',
        'add_new' => 'Add New',
        'add_new_item' => 'Add New Testimonial',
        'edit_item' => 'Edit Testimonial',
        'new_item' => 'New Testimonial',
        'view_item' => 'View Testimonial',
        'search_items' => 'Search Testimonials',
        'not_found' => 'No Testimonials found',
        'not_found_in_trash' => 'No Testimonials in the trash',
        'parent_item_colon' => ''
    );
    register_post_type('testimonials', array(
        'labels' => $labels,
        'public' => true,
        'publicly_queryable' => true,
        'show_ui' => true,
        'exclude_from_search' => false,
        'query_var' => true,
        'rewrite' => true,
        'capability_type' => 'post',
        'has_archive' => false,
        'hierarchical' => false,
        'menu_position' => 4,
        'supports' => array(
            'title',
            'editor',
            'thumbnail'
        ),
        'register_meta_box_cb' => 'testimonials_meta_boxes' // Callback function for custom metaboxes
    ));
}
function testimonials_meta_boxes()
{
    add_meta_box('testimonials_form', 'Testimonial Details', 'testimonials_form', 'testimonials', 'normal', 'high');
}
function testimonials_form()
{
    $post_id          = get_the_ID();
    $testimonial_data = get_post_meta($post_id, '_testimonial', true);
    $client_name      = (empty($testimonial_data['client_name'])) ? '' : $testimonial_data['client_name'];
    $source           = (empty($testimonial_data['source'])) ? '' : $testimonial_data['source'];
    $link             = (empty($testimonial_data['link'])) ? '' : $testimonial_data['link'];
	$star             = (empty($testimonial_data['star'])) ? '' : $testimonial_data['star'];
    wp_nonce_field('testimonials', 'testimonials');
?>

<p>
  <label>Client's Name (optional)</label>
  <br />
  <input type="text" value="<?php
    echo $client_name;
?>" name="testimonial[client_name]" size="40" />
</p>
<p>
  <label>Business/Site Name (optional)</label>
  <br />
  <input type="text" value="<?php
    echo $source;
?>" name="testimonial[source]" size="40" />
</p>
<p>
  <label>Link (optional)</label>
  <br />
  <input type="text" value="<?php
    echo $link;
?>" name="testimonial[link]" size="40" />
</p>
<p>
  <label>Ratings (optional) 1-5</label>
  <br />
  <input type="text" value="<?php
    echo $star;
?>" name="testimonial[star]" size="40" />


</p>
<?php
}
add_action('save_post', 'testimonials_save_post');
function testimonials_save_post($post_id)
{
    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE)
        return;
    if (!empty($_POST['testimonials']) && !wp_verify_nonce($_POST['testimonials'], 'testimonials'))
        return;
    if (!empty($_POST['post_type']) && 'page' == $_POST['post_type']) {
        if (!current_user_can('edit_page', $post_id))
            return;
    } else {
        if (!current_user_can('edit_post', $post_id))
            return;
    }
    if (!wp_is_post_revision($post_id) && 'testimonials' == get_post_type($post_id)) {
        remove_action('save_post', 'testimonials_save_post');
        add_action('save_post', 'testimonials_save_post');
    }
    if (!empty($_POST['testimonial'])) {
        $testimonial_data['client_name'] = (empty($_POST['testimonial']['client_name'])) ? '' : sanitize_text_field($_POST['testimonial']['client_name']);
        $testimonial_data['source']      = (empty($_POST['testimonial']['source'])) ? '' : sanitize_text_field($_POST['testimonial']['source']);
        $testimonial_data['link']        = (empty($_POST['testimonial']['link'])) ? '' : esc_url($_POST['testimonial']['link']);
		$testimonial_data['star']        = (empty($_POST['testimonial']['star'])) ? '' : sanitize_text_field($_POST['testimonial']['star']);
        update_post_meta($post_id, '_testimonial', $testimonial_data);
    } else {
        delete_post_meta($post_id, '_testimonial');
    }
}
add_filter('manage_edit-testimonials_columns', 'testimonials_edit_columns');
function testimonials_edit_columns($columns)
{
    $columns = array(
        'cb' => '<input type="checkbox" />',
        'testimonial-photo' => 'Photo',
        'title' => 'Title',
        'testimonial' => 'Testimonial',
        'testimonial-star' => 'Ratings',
        'date' => 'Date'
    );
    return $columns;
}
add_action('manage_posts_custom_column', 'testimonials_columns', 10, 2);
function testimonials_columns($column, $post_id)
{
    $testimonial_data = get_post_meta($post_id, '_testimonial', true);
    switch ($column) {
        case 'testimonial-photo':
            if (has_post_thumbnail()):
                $banner_image_url = wp_get_attachment_image_src(get_post_thumbnail_id($post->ID), 'thumbnail');
                echo '<img src="' . $banner_image_url[0] . '" width="80px" height="80px">';
            endif;
            break;
        case 'testimonial':
            the_excerpt();
            break;
        case 'testimonial-client-name':
            if (!empty($testimonial_data['client_name']))
                echo $testimonial_data['client_name'];
            break;
        case 'testimonial-source':
            if (!empty($testimonial_data['source']))
                echo $testimonial_data['source'];
            break;
        case 'testimonial-link':
            if (!empty($testimonial_data['link']))
                echo $testimonial_data['link'];
            break;
		        case 'testimonial-star':
            if (!empty($testimonial_data['star']))
                echo $testimonial_data['star'].' Star(s)';
            break;
    }
}
/**



* Display a testimonial



*



* @param  int $post_per_page  The number of testimonials you want to display



* @param  string $orderby  The order by setting  https://codex.wordpress.org/Class_Reference/WP_Query#Order_.26_Orderby_Parameters



* @param  array $testimonial_id  The ID or IDs of the testimonial(s), comma separated



*



* @return  string  Formatted HTML



*/
function get_testimonial($posts_per_page = 1, $orderby = 'none', $testimonial_id = null)
{
    $args = array(
        'posts_per_page' => (int) $posts_per_page,
        'post_type' => 'testimonials',
        'orderby' => $orderby,
        'no_found_rows' => true
    );
    if ($testimonial_id)
        $args['post__in'] = array(
            $testimonial_id
        );
    $query        = new WP_Query($args);
    $testimonials = '';
    if ($query->have_posts()) {
        while ($query->have_posts()):
            $query->the_post();
            $post_id          = get_the_ID();
            $testimonial_data = get_post_meta($post_id, '_testimonial', true);
            $client_name      = (empty($testimonial_data['client_name'])) ? '' : $testimonial_data['client_name'];
            $source           = (empty($testimonial_data['source'])) ? '' : ' - ' . $testimonial_data['source'];
            $link             = (empty($testimonial_data['link'])) ? '' : $testimonial_data['link'];
			$star             = (empty($testimonial_data['star'])) ? '' : $testimonial_data['star'];
            $cite             = ($link) ? '<a href="' . esc_url($link) . '" target="_blank">' . $client_name . $source . '</a>' : $client_name . $source;
            $banner_image_url = wp_get_attachment_image_src(get_post_thumbnail_id($post->ID), 'thumbnail');
            $testimonials .= '<div class="testimonial my-post">';
            $testimonials .= '<header class="entry-header"  itemprop="name headline"><h2 class="entry-title"><a href="' . get_the_permalink() . '">' . get_the_title() . '</a></h2>';
            $testimonials .= '<div class="entry-meta">Submitted by: ' . $cite . '</div></header>';
            $testimonials .= '<div class="entry-summary">';
            if (has_post_thumbnail()):
                $testimonials .= '<img src="' . $banner_image_url[0] . '" class="alignleft size-thumbnail">';
            endif;
            $testimonials .= get_the_excerpt();
            $testimonials .= '</div></div>';
        endwhile;
        wp_reset_postdata();
    }
    return $testimonials;
}
add_shortcode('testimonial', 'testimonial_shortcode');
/**



* Shortcode to display testimonials



*



* [testimonial posts_per_page="1" orderby="none" testimonial_id=""]



*/
function testimonial_shortcode($atts)
{
    extract(shortcode_atts(array(
        'posts_per_page' => '-1',
        'orderby' => 'none',
        'testimonial_id' => ''
    ), $atts));
    return get_testimonial($posts_per_page, $orderby, $testimonial_id);
}
?>